﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
namespace MixinSample {
  #region --- mixin automatically generated.
  partial class ImageButton: Button
  {
        #region フィールド
        private PropertyInfo checkedPropertyInfo = null;
        private bool isHovered = false;
        private bool isChecked = false;
        #endregion

        #region メソッド
        /// <summary>
        /// コンストラクタです。
        /// </summary>
        public void ImageButtonModuleConstructor()
        {
            // MouseEnter イベントに処理を追加します。
            MouseEnter += delegate
            {
                // ホバー状態を表すフラグを立てます。
                isHovered = true;
                // 再描画する。
                Invalidate();
            };
            // MouseLeave イベントに処理を追加します。
            MouseLeave += delegate
            {
                // ホバー状態を表すフラグを落とします。
                isHovered = false;
                // 再描画します。
                Invalidate();
            };

            // Checked プロパティを取得します。
            checkedPropertyInfo = GetType().GetProperty("Checked");
            // Checked プロパティが存在するならば、
            if (checkedPropertyInfo != null)
            {
                // CheckedChanged イベントを取得します。
                EventInfo checkedEventInfo = GetType().GetEvent("CheckedChanged");
                // CheckedChanged イベントに処理を追加します。
                checkedEventInfo.AddEventHandler(this, new EventHandler(delegate
                {
                    // チェック状態を表すフラグを取得します。
                    isChecked = (bool)checkedPropertyInfo.GetValue(this, null);
                    // 再描画します。
                    Invalidate();
                }));
            }
        }

        /// <summary>
        /// 状態を元に描画すべきイメージを返します。
        /// </summary>
        /// <returns>描画すべきイメージ。</returns>
        private Image GetImage()
        {
            // チェック状態ならば、
            if (isChecked)
            {
                // checkedImage を返します。
                return checkedImage;
            }
            // ホバー状態ならば、
            if (isHovered)
            {
                // hoveredImage を返します。
                return hoveredImage;
            }
            // 継承元の Image を返します。
            return Image;
        }
        #endregion

        #region オーバーライドされたメソッド
        protected override void OnPaint(PaintEventArgs pevent)
        {
            // グラフィックスを取得します。
            Graphics g = pevent.Graphics;

            // 描画すべきイメージを取得します。
            Image image = GetImage();

            // 描画すべきイメージが存在するのであれば、
            if (image != null)
            {
                // コントロールが有効であれば、
                if (this.Enabled)
                {
                    // 通常の描画を行います。
                    g.DrawImage(image, 0, 0);
                }
                else
                {
                    // 無効状態の描画を行います。
                    ControlPaint.DrawImageDisabled(g, image, 0, 0, Color.Black);
                }
            }
            else
            {
                // 黒で塗りつぶします。
                g.FillRectangle(Brushes.Black, pevent.ClipRectangle);
            }
        }

        protected override void OnPaintBackground(PaintEventArgs pevent)
        {
            // 何もしません。
        }
        #endregion

        #region プロパティ
        private Image hoveredImage = null;
        /// <summary>
        /// ホバー状態の時にボタン コントロールに表示されるイメージを取得または設定します。
        /// </summary>
        [Category("表示")]
        [Description("ホバー状態の時にコントロールに表示されるイメージです。")]
        [DefaultValue(null)]
        public Image HoveredImage
        {
            get
            {
                return hoveredImage;
            }
            set
            {
                hoveredImage = value;
            }
        }

        private Image checkedImage = null;
        /// <summary>
        /// チェック状態の時にボタン コントロールに表示されるイメージを取得または設定します。
        /// </summary>
        [Category("表示")]
        [Description("チェック状態の時にコントロールに表示されるイメージです。")]
        [DefaultValue(null)]
        public Image CheckedImage
        {
            get
            {
                return checkedImage;
            }
            set
            {
                checkedImage = value;
            }
        }
        #endregion
  }
  #endregion --- mixin automatically generated.
  #region --- mixin automatically generated.
  partial class ImageCheckBox: CheckBox
  {
        #region フィールド
        private PropertyInfo checkedPropertyInfo = null;
        private bool isHovered = false;
        private bool isChecked = false;
        #endregion

        #region メソッド
        /// <summary>
        /// コンストラクタです。
        /// </summary>
        public void ImageButtonModuleConstructor()
        {
            // MouseEnter イベントに処理を追加します。
            MouseEnter += delegate
            {
                // ホバー状態を表すフラグを立てます。
                isHovered = true;
                // 再描画する。
                Invalidate();
            };
            // MouseLeave イベントに処理を追加します。
            MouseLeave += delegate
            {
                // ホバー状態を表すフラグを落とします。
                isHovered = false;
                // 再描画します。
                Invalidate();
            };

            // Checked プロパティを取得します。
            checkedPropertyInfo = GetType().GetProperty("Checked");
            // Checked プロパティが存在するならば、
            if (checkedPropertyInfo != null)
            {
                // CheckedChanged イベントを取得します。
                EventInfo checkedEventInfo = GetType().GetEvent("CheckedChanged");
                // CheckedChanged イベントに処理を追加します。
                checkedEventInfo.AddEventHandler(this, new EventHandler(delegate
                {
                    // チェック状態を表すフラグを取得します。
                    isChecked = (bool)checkedPropertyInfo.GetValue(this, null);
                    // 再描画します。
                    Invalidate();
                }));
            }
        }

        /// <summary>
        /// 状態を元に描画すべきイメージを返します。
        /// </summary>
        /// <returns>描画すべきイメージ。</returns>
        private Image GetImage()
        {
            // チェック状態ならば、
            if (isChecked)
            {
                // checkedImage を返します。
                return checkedImage;
            }
            // ホバー状態ならば、
            if (isHovered)
            {
                // hoveredImage を返します。
                return hoveredImage;
            }
            // 継承元の Image を返します。
            return Image;
        }
        #endregion

        #region オーバーライドされたメソッド
        protected override void OnPaint(PaintEventArgs pevent)
        {
            // グラフィックスを取得します。
            Graphics g = pevent.Graphics;

            // 描画すべきイメージを取得します。
            Image image = GetImage();

            // 描画すべきイメージが存在するのであれば、
            if (image != null)
            {
                // コントロールが有効であれば、
                if (this.Enabled)
                {
                    // 通常の描画を行います。
                    g.DrawImage(image, 0, 0);
                }
                else
                {
                    // 無効状態の描画を行います。
                    ControlPaint.DrawImageDisabled(g, image, 0, 0, Color.Black);
                }
            }
            else
            {
                // 黒で塗りつぶします。
                g.FillRectangle(Brushes.Black, pevent.ClipRectangle);
            }
        }

        protected override void OnPaintBackground(PaintEventArgs pevent)
        {
            // 何もしません。
        }
        #endregion

        #region プロパティ
        private Image hoveredImage = null;
        /// <summary>
        /// ホバー状態の時にボタン コントロールに表示されるイメージを取得または設定します。
        /// </summary>
        [Category("表示")]
        [Description("ホバー状態の時にコントロールに表示されるイメージです。")]
        [DefaultValue(null)]
        public Image HoveredImage
        {
            get
            {
                return hoveredImage;
            }
            set
            {
                hoveredImage = value;
            }
        }

        private Image checkedImage = null;
        /// <summary>
        /// チェック状態の時にボタン コントロールに表示されるイメージを取得または設定します。
        /// </summary>
        [Category("表示")]
        [Description("チェック状態の時にコントロールに表示されるイメージです。")]
        [DefaultValue(null)]
        public Image CheckedImage
        {
            get
            {
                return checkedImage;
            }
            set
            {
                checkedImage = value;
            }
        }
        #endregion
  }
  #endregion --- mixin automatically generated.
  #region --- mixin automatically generated.
  partial class ImageRadioButton: RadioButton
  {
        #region フィールド
        private PropertyInfo checkedPropertyInfo = null;
        private bool isHovered = false;
        private bool isChecked = false;
        #endregion

        #region メソッド
        /// <summary>
        /// コンストラクタです。
        /// </summary>
        public void ImageButtonModuleConstructor()
        {
            // MouseEnter イベントに処理を追加します。
            MouseEnter += delegate
            {
                // ホバー状態を表すフラグを立てます。
                isHovered = true;
                // 再描画する。
                Invalidate();
            };
            // MouseLeave イベントに処理を追加します。
            MouseLeave += delegate
            {
                // ホバー状態を表すフラグを落とします。
                isHovered = false;
                // 再描画します。
                Invalidate();
            };

            // Checked プロパティを取得します。
            checkedPropertyInfo = GetType().GetProperty("Checked");
            // Checked プロパティが存在するならば、
            if (checkedPropertyInfo != null)
            {
                // CheckedChanged イベントを取得します。
                EventInfo checkedEventInfo = GetType().GetEvent("CheckedChanged");
                // CheckedChanged イベントに処理を追加します。
                checkedEventInfo.AddEventHandler(this, new EventHandler(delegate
                {
                    // チェック状態を表すフラグを取得します。
                    isChecked = (bool)checkedPropertyInfo.GetValue(this, null);
                    // 再描画します。
                    Invalidate();
                }));
            }
        }

        /// <summary>
        /// 状態を元に描画すべきイメージを返します。
        /// </summary>
        /// <returns>描画すべきイメージ。</returns>
        private Image GetImage()
        {
            // チェック状態ならば、
            if (isChecked)
            {
                // checkedImage を返します。
                return checkedImage;
            }
            // ホバー状態ならば、
            if (isHovered)
            {
                // hoveredImage を返します。
                return hoveredImage;
            }
            // 継承元の Image を返します。
            return Image;
        }
        #endregion

        #region オーバーライドされたメソッド
        protected override void OnPaint(PaintEventArgs pevent)
        {
            // グラフィックスを取得します。
            Graphics g = pevent.Graphics;

            // 描画すべきイメージを取得します。
            Image image = GetImage();

            // 描画すべきイメージが存在するのであれば、
            if (image != null)
            {
                // コントロールが有効であれば、
                if (this.Enabled)
                {
                    // 通常の描画を行います。
                    g.DrawImage(image, 0, 0);
                }
                else
                {
                    // 無効状態の描画を行います。
                    ControlPaint.DrawImageDisabled(g, image, 0, 0, Color.Black);
                }
            }
            else
            {
                // 黒で塗りつぶします。
                g.FillRectangle(Brushes.Black, pevent.ClipRectangle);
            }
        }

        protected override void OnPaintBackground(PaintEventArgs pevent)
        {
            // 何もしません。
        }
        #endregion

        #region プロパティ
        private Image hoveredImage = null;
        /// <summary>
        /// ホバー状態の時にボタン コントロールに表示されるイメージを取得または設定します。
        /// </summary>
        [Category("表示")]
        [Description("ホバー状態の時にコントロールに表示されるイメージです。")]
        [DefaultValue(null)]
        public Image HoveredImage
        {
            get
            {
                return hoveredImage;
            }
            set
            {
                hoveredImage = value;
            }
        }

        private Image checkedImage = null;
        /// <summary>
        /// チェック状態の時にボタン コントロールに表示されるイメージを取得または設定します。
        /// </summary>
        [Category("表示")]
        [Description("チェック状態の時にコントロールに表示されるイメージです。")]
        [DefaultValue(null)]
        public Image CheckedImage
        {
            get
            {
                return checkedImage;
            }
            set
            {
                checkedImage = value;
            }
        }
        #endregion
  }
  #endregion --- mixin automatically generated.
}
